# servctrl.tcl for netbots.tcl v3.30
# designed to work with eggdrop 1.3.28 or higher
# Johoho's Eggdrop Page - http://johoho.tuts.net/eggdrop/
# restricted.tcl copyright (c) 1999,2000 by johoho <johoho@hojo-net.de>
# netbots.tcl copyright (c) 1998-2000 by slennox <slennox@egghelp.org>

## servctrl.tcl component script v2.10.0, 30.04.2000 ##

# History:
# 
# v1.00.1, 22.04.2000 - initial release
# v2.00.0, 28.04.2000 - extended the script
# v2.10.0, 30.04.2000 - added an option to make not to crash on startup

# thanks slennox for netbots.tcl
# thanks to Abraham for ideas/bugreports/feeback

set check-mode-r 0

if {![file exists $sc_file]} {
  set f [open $sc_file w]
  close $f
}

if {![info exists sc_raw_klined]} {
  set sc_raw_klined 465
}
if {![info exists sc_raw_restricted]} {
  set sc_raw_restricted 484
}
if {![info exists sc_raw_unauthorized]} {
  set sc_raw_unauthorized 433
}

if {![info exists sc_starttimeout]} {
  set sc_starttimeout 3
}

###

proc sc_helpidx {hand chan idx} {
  if {![matchattr $hand m|m $chan]} {return 0}
  putidx $idx "servctrl.tcl commands"
  putidx $idx " For channel masters:"
  putidx $idx "  rmserv        servprob"
  putidx $idx "  klines        restricted    unauthorized"
  putidx $idx " "
  return 1
}

proc sc_help {hand chan idx cmd} {
global sc_klined sc_restricted sc_unauthorized
  if {[matchattr $hand m|m $chan]} {
    switch -exact -- $cmd {
      "unauthorized" {
        putidx $idx "# unauthorized"
        putwrap $idx 3 "Lists servers on which bot is unauthorized."
        if {!$sc_unauthorized} {
        putwrap $idx 3 "Checking of unauthorized status is disabled, so that command too."
        }
        return 1
      }
      "klines" {
        putidx $idx "# klines"
        putwrap $idx 3 "Lists servers on which bot is klined."
        if {!$sc_klined} {
        putwrap $idx 3 "Checking of unauthorized status is disabled, so that command too."
        }
        return 1
      }
      "restricted" {
        putidx $idx "# restricted"
        putwrap $idx 3 "Lists servers on which bot is restricted."
        if {!$sc_restricted} {
        putwrap $idx 3 "Checking of unauthorized status is disabled, so that command too."
        }
        return 1
      }
      "servprob" {
        putidx $idx "# servprob"
        putwrap $idx 3 "Lists servers on which bot is unauthorized, restricted or klined."
        return 1
      }
      "rmserv" {
        putidx $idx "# rmserv <option> <server> \[bot\]"
        putwrap $idx 3 "Removes a server from store file file according by status."
        putwrap $idx 3 "option: -r -k -u"
        putwrap $idx 3 "server: irc.server.org"
        putwrap $idx 3 "bot:    optional bot where the command should be executed"
        putwrap $idx 3 " "
        putwrap $idx 3 "Use:"
        putwrap $idx 3 "-k for servers where the bot is k-lined,"
        putwrap $idx 3 "-r for servers where the connection by the bot is restricted and"
        putwrap $idx 3 "-u for servers where the connection by the bot is unauthorized"
        return 1
      }
    }
  }
  return 0
}

lappend nb_helpidx "sc_helpidx"
set nb_help(unauthorized) "sc_help"
set nb_help(klines) "sc_help"
set nb_help(restricted) "sc_help"
set nb_help(servprob) "sc_help"
set nb_help(rmserv) "sc_help"

proc sc_klined_connection {from signal arg} {
global botnick sc_note sc_file sc_raw_klined
  set f [open $sc_file r]
  while {![eof $f]} {
	set line [gets $f]
    if {[regexp -- "^$sc_raw_klined:" $line]} {
      regsub -- "$sc_raw_klined:" $line "" serv
      if {$serv == $from} {
        set already 1
        break
      }
    }
  }
  close $f
  if {![info exists already]} {
    set f [open $sc_file a]
    puts $f "$sc_raw_klined:$from"
    close $f
  }
  sc_rmserver $from
  jump
  putlog "Ooops, I am k-lined on $from"
  foreach recipient $sc_note {
    if {[validuser $recipient]} {
      sendnote SERVCTRL $recipient "I ($botnick) am k-lined on $from"
    }
  }
  return 1
}

proc sc_restricted_connection {from signal arg} {
global botnick sc_note sc_file sc_raw_restricted
  set f [open $sc_file r]
  while {![eof $f]} {
	set line [gets $f]
    if {[regexp -- "^$sc_raw_restricted:" $line]} {
      regsub -- "$sc_raw_restricted:" $line "" serv
      if {$serv == $from} {
        set already 1
        break
      }
    }
  }
  close $f
  if {![info exists already]} {
    set f [open $sc_file a]
    puts $f "$sc_raw_restricted:$from"
    close $f
  }
  sc_rmserver $from
  jump
  putlog "Ooops, I am restricted on $from"
  foreach recipient $sc_note {
    if {[validuser $recipient]} {
      sendnote SERVCTRL $recipient "I ($botnick) am restricted on $from"
    }
  }
  return 1
}

proc sc_unauthorized_connection {from signal arg} {
global botnick sc_note sc_file sc_raw_unauthorized
  set f [open $sc_file r]
  while {![eof $f]} {
	set line [gets $f]
    if {[regexp -- "^$sc_raw_unauthorized:" $line]} {
      regsub -- "$sc_raw_unauthorized:" $line "" serv
      if {$serv == $from} {
        set already 1
        break
      }
    }
  }
  close $f
  if {![info exists already]} {
    set f [open $sc_file a]
    puts $f "$sc_raw_unauthorized:$from"
    close $f
  }
  sc_rmserver $from
  jump
  putlog "Ooops, I am unauthorized on $from"
  foreach recipient $sc_note {
    if {[validuser $recipient]} {
      sendnote SERVCTRL $recipient "I ($botnick) am unauthorized on $from"
    }
  }
  return 1
}

proc sc_rmserver_help1 {bla args} {
global servers
  set tmp [lindex $bla 0]
  if {$tmp == ""} { return 0 }
  set line "$tmp*"
  for {set j 0} {$j >= 0} {incr j} {
    set x [lsearch $servers $line]
    if {$x >= 0} {
      set servers [lreplace $servers $x $x]
    }
    if {$x < 0} {
      if {$j >= 0} {
        putlog "SERVCTRL: Removed server $tmp from server cycling queue"
      }
      break
    }
  }
}

proc sc_rmserver {torem args} {
global servers sc_file sc_note sc_raw_klined sc_raw_restricted sc_starttimeout sc_raw_unauthorized
  set kservers {}
  if {$torem == "x"} {
    set f [open $sc_file r]
    while {![eof $f]} {
      set line [gets $f]
      set tmp ""
      if {[regexp -- "^$sc_raw_unauthorized:" $line]} {
        regsub -- "$sc_raw_unauthorized:" $line "" tmp
      } elseif {[regexp -- "^$sc_raw_klined:" $line]} {
        regsub -- "$sc_raw_klined:" $line "" tmp
      } elseif {[regexp -- "^$sc_raw_restricted:" $line]} {
        regsub -- "$sc_raw_restricted:" $line "" tmp
      }
      if {[eof $f]} {
        utimer $sc_starttimeout [sc_rmserver_help1 $tmp]
        break
      } else {
        sc_rmserver_help1 $tmp
      }
      unset tmp
    }
    close $f
  } else {
    sc_rmserver_help1 $torem
  }
}

sc_rmserver "x"

proc sc_dccrm_server {hand idx arg} {
global sc_raw_klined sc_raw_restricted sc_raw_unauthorized
  set option [lindex $arg 0]
  set serv [lindex $arg 1]
  set bot [lindex $arg 2]
  switch -exact -- $option {
    "-k" {set file "klined"}
    "-r" {set file "restricted"}
    "-u" {set file "unauthorized"}
    "default" {set file "nothing"}
  }
  if {[string match -* $option]} {
    if {$file == "nothing"} {
      putidx $idx "Valid options are: -k, -r, -u" ; return 0
    } else {
      if {$serv == ""} {
        putidx $idx "You must specify a server to remove" ; return 0
      } elseif {$file == "klined"} {
        regsub -- "$serv" $serv "$sc_raw_klined:$serv" tmp
      } elseif {$file == "restricted"} {
        regsub -- "$serv" $serv "$sc_raw_restricted:$serv" tmp
      } elseif {$file == "unauthorized"} {
        regsub -- "$serv" $serv "$sc_raw_unauthorized:$serv" tmp
      }
      set $serv $tmp
    }
  } else {
    putdcc $idx "Usage: .rmserv <option> <server> \[bot\]"
    return 0   
  }
  if {$bot != ""} {
    if {![islinked $bot]} {
      putidx $idx "$bot is not linked." ; return 0
    }
    putcmdlog "#$hand# rmserv $option $serv $bot"
    nb_sendcmd $bot sc_netrm_server_help1 "$hand $idx $option $serv"
    return 1
  } else {
    sc_dccrm_server_help1 "x" "$hand $idx $file $serv"
    return 1
  }
}

proc sc_dccrm_server_help1 {bla arg} {
global sc_file sc_raw_klined sc_raw_restricted sc_raw_unauthorized
  set hand [lindex $arg 0]
  set idx [lindex $arg 1]
  set file [lindex $arg 2]
  set serv [lindex $arg 3]
  set tmparray {}
  set f [open $sc_file r]
  while {![eof $f]} {
    set line [gets $f]
    if {[eof $f]} {break}
    set tmparray [lappend tmparray [string trim $line]]
  }
  close $f
  set tmp "$serv"
  if {$file == "klined"} {
    regsub -- "$tmp" $tmp "$sc_raw_klined:$tmp" serv
  } elseif {$file == "restricted"} {
    regsub -- "$tmp" $tmp "$sc_raw_restricted:$tmp" serv
  } elseif {$file == "unauthorized"} {
    regsub -- "$tmp" $tmp "$sc_raw_unauthorized:$tmp" serv
  }
  set serv "$serv*"
  for {set j 0} {$j >= 0} {incr j} {
    set x [lsearch $tmparray $serv]
    if {$x >= 0} {
      set tmparray [lreplace $tmparray $x $x]
      set z 1
    }
    if {$x < 0} {
      if {$j >= 0} {
        break
      }
    }
  }
  if {($x == "-1") && ![info exists z] } {
    putlog "$tmp wasn't found in store file"
    return 0 
  } else {
    putlog "rewriting $sc_file ..."
    exec mv $sc_file $sc_file.old
    set f [open $sc_file w]
    foreach bla $tmparray {
      puts $f $bla
    } 
    close $f
    putlog "Done. $tmp was removed from store file"
    return 1
  }
}

proc sc_netrm_server_help1 {frombot arg} {
  putcmdlog "#$hand@$frombot# rmserv $option $serv"
  sc_dccrm_server_help1 "x" "$arg"
}

proc sc_list_klined {hand idx args} {
global sc_file sc_raw_klined
  set kservers { }
  set f [open $sc_file r]
  while {![eof $f]} {
    set line [gets $f]
    if {[regexp -- "^$sc_raw_klined:" $line]} {
      regsub -- "$sc_raw_klined:" $line "" tmp
      set kservers [lappend kservers $tmp]
    }
  }
  close $f
  putdcc $idx "Servers on which I'm k-lined:"
  if {[llength $kservers] == 0} {
    putdcc $idx "  - none"
  } else {
    foreach tmp $kservers {
      putdcc $idx "  - $tmp"
    }
  }
}

proc sc_list_restricted {hand idx args} {
global sc_file sc_raw_restricted
  set kservers { }
  set f [open $sc_file r]
  while {![eof $f]} {
	set line [gets $f]
    if {[regexp -- "^$sc_raw_restricted:" $line]} {
      regsub -- "$sc_raw_restricted:" $line "" tmp
      set kservers [lappend kservers $tmp]
    }
  }
  close $f
  putdcc $idx "Servers on which I'm restricted:"
  if {[llength $kservers] == 0} {
    putdcc $idx "  - none"
  } else {
    foreach tmp $kservers {
      putdcc $idx "  - $tmp"
    }
  }
}

proc sc_list_unauthorized {hand idx args} {
global sc_file sc_raw_unauthorized
  set kservers { }
  set f [open $sc_file r]
  while {![eof $f]} {
	set line [gets $f]
    if {[regexp -- "^$sc_raw_unauthorized:" $line]} {
      regsub -- "$sc_raw_unauthorized:" $line "" tmp
      set kservers [lappend kservers $tmp]
    }
  }
  close $f
  putdcc $idx "Servers on which I'm unauthorized:"
  if {[llength $kservers] == 0} {
    putdcc $idx "  - none"
  } else {
    foreach tmp $kservers {
      putdcc $idx "  - $tmp"
    }
  }
}

proc sc_list_all {hand idx args} {
global sc_klined sc_restricted sc_unauthorized
  putdcc $idx " "
  if {$sc_klined} {
    sc_list_klined $hand $idx $args
    putdcc $idx " "
  }
  if {$sc_restricted} {
    sc_list_restricted $hand $idx $args
    putdcc $idx " "
  }
  if {$sc_unauthorized} {
    sc_list_unauthorized $hand $idx $args
    putdcc $idx " "
  }
  if {(!$sc_klined) && (!$sc_restricted) && (!$sc_unauthorized)} {
    putwrap $idx 3 "You disabled all settings in your config. Therefore I can't display anything here."
    putdcc $idx " "
  }
  return 1
}

set nb_netcmds(sc_netrm_server_help1) "sc_netrm_server_help1"

bind dcc m|m servprob sc_list_all
bind dcc m|m rmserv sc_dccrm_server

bind dcc m|m klines sc_list_klined
bind raw * $sc_raw_klined sc_klined_connection
if {!$sc_klined} {
  unbind dcc m|m klines sc_list_klined
  unbind raw * $sc_raw_klined sc_klined_connection
}

bind dcc m|m restricted sc_list_restricted
bind raw * $sc_raw_restricted sc_restricted_connection
if {!$sc_restricted} {
  unbind dcc m|m restricted sc_list_restricted
  unbind raw * $sc_raw_restricted sc_restricted_connection
}

bind dcc m|m unauthorized sc_list_unauthorized
bind raw * $sc_raw_unauthorized sc_unauthorized_connection
if {!$sc_unauthorized} {
  unbind dcc m|m unauthorized sc_list_unauthorized
  unbind raw * $sc_raw_unauthorized sc_unauthorized_connection
}
